<?php

namespace Drupal\formassembly\Form;

use Drupal\Core\Form\ConfigFormBaseTrait;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\formassembly\ApiAuthorize;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * FormAssembly entity settings form.
 *
 * @author Shawn P. Duncan <code@sd.shawnduncan.org>
 *
 * Copyright 2018 by Shawn P. Duncan.  This code is
 * released under the GNU General Public License.
 * Which means that it is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * http://www.gnu.org/licenses/gpl.html
 * @package Drupal\formassembly
 */
class FormAssemblyEntitySettingsForm extends FormBase {

  use ConfigFormBaseTrait;

  /**
   * Injected authorization service.
   *
   * @var \Drupal\formassembly\ApiAuthorize
   */
  protected $authorize;

  /**
   * FormAssemblyEntitySettingsForm constructor.
   *
   * @param \Drupal\formassembly\ApiAuthorize $authorize
   *   Injected authorization service.
   */
  public function __construct(ApiAuthorize $authorize) {
    $this->authorize = $authorize;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('formassembly.authorize')
    );
  }

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'FormAssemblyEntity_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'formassembly.api.oauth',
    ];
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Config\Config $formassembly_config */
    $formassembly_config = $this->config('formassembly.api.oauth');
    $formassembly_config->set('cid', $form_state->getValue('cid'))
      ->set('secret', $form_state->getValue('secret'))
      ->set('endpoint', $form_state->getValue('endpoint'))
      ->set('instance', $form_state->getValue('instance'))
      ->set('admin_index', $form_state->getValue('admin_index'))
      ->save();
    $sync = $form_state->getValue('batch_sync');
    $reauthorize = $form_state->getValue('reauthorize');
    if ($reauthorize || !$this->authorize->isAuthorized()) {
      $form_state->setRedirect('fa_form.authorize');
    }
    if ($sync) {
      // Setup a batch.
      $batch = [
        'operations' => [
          ['formassembly_batch_get_forms', []],
        ],
        'finished' => 'formassembly_batch_finished',
        'title' => 'Request Forms Data from FormAssembly',
        'init_message' => 'Contacting FormAssembly',
      ];
      batch_set($batch);
    }
  }

  /**
   * Defines the settings form for FormAssembly Form entities.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   Form definition array.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $oauth_config = $this->config('formassembly.api.oauth');
    $form = [];
    $form['overrides'] = [
      '#type' => 'item',
      '#title' => 'Overrides',
      '#markup' => $this->t('These values may be overridden via a file included in <em>settings.php</em> for added security.  Fields on this form are disabled when overrides are detected. The overridden values are never displayed here for security.'),
    ];
    $form['cid'] = [
      '#type' => 'textfield',
      '#required' => FALSE,
      '#title' => t('FormAssembly OAuth Client ID'),
      '#description' => t('The client ID.'),
      '#default_value' => $oauth_config->get('cid'),
      '#disabled' => $this->isOverridden('cid'),
    ];

    $form['secret'] = [
      '#type' => 'textfield',
      '#required' => FALSE,
      '#title' => t('FormAssembly OAuth Client Secret'),
      '#description' => t('The client secret.'),
      '#default_value' => $oauth_config->get('secret'),
      '#disabled' => $this->isOverridden('secret'),
    ];

    $form['endpoint'] = [
      '#type' => 'url',
      '#title' => t('FormAssembly Url'),
      '#required' => TRUE,
      '#description' => t('The url of your instance. Examples: https://developer.formassembly.com, https://app.formassembly.com, https://instance_name.tfaforms.net, https://your_server/path/to/formassembly'),
      '#default_value' => $oauth_config->get('endpoint'),
      '#disabled' => $this->isOverridden('endpoint'),
    ];

    $form['admin_index'] = [
      '#type' => 'checkbox',
      '#required' => FALSE,
      '#title' => t('Admin Index'),
      '#description' => t('Return a list of all forms in the FormAssembly instance. Only applies to Enterprise level instances. The authentication tokens entered above must be from an admin-level user'),
      '#default_value' => $oauth_config->get('admin_index'),
      '#disabled' => $this->isOverridden('admin_index'),
    ];
    $documentation = Url::fromUri('http://docs.drush.org/en/master/cron/');
    $form['batch_sync'] = [
      '#type' => 'checkbox',
      '#title' => t('Sync now'),
      '#description' => t('Sync forms after submitting this form. <em>Disabled if authorization is not complete.</em>'),
      '#default_value' => FALSE,
      '#disabled' => !$this->authorize->isAuthorized(),
      '#suffix' => '<div>' . t('To sync on cron, place the drush command %command into your crontab.  See @url for more information.',
          [
            '%command' => 'drush fa-sync',
            '@url' => $this->getLinkGenerator()
              ->generate('Running Drupal cron tasks from Drush',
                $documentation),
          ]) . '</div>',
    ];

    $form['reauthorize'] = [
      '#type' => 'checkbox',
      '#title' => t('Reauthorize'),
      '#description' => t('Reauthorize this application and get a new access token.'),
      '#default_value' => FALSE,
    ];

    $form['save'] = [
      '#type' => 'submit',
      '#value' => t('Submit'),
    ];

    return $form;
  }

  /**
   * Check if config variable is overridden by the settings.php.
   *
   * @param string $name
   *   The settings key.
   *
   * @return bool
   *   If the config on this form is overridden.
   */
  protected function isOverridden($name) {
    $original = $this->configFactory->getEditable('formassembly.api.oauth')
      ->get($name);
    $current = $this->configFactory->get('formassembly.api.oauth')
      ->get($name);
    return $original != $current;
  }

}
